<?php
/**
 * Search Logger - Main Controller
 *
 * @package   SearchLogger
 * @copyright (c) 2025 salvocortesiano
 * @license   GNU General Public License, version 2 (GPL-2.0)
 */

namespace salvocortesiano\searchlogger\controller;

/**
 * Main controller
 */
class main
{
    /** @var \phpbb\config\config */
    protected $config;

    /** @var \phpbb\db\driver\driver_interface */
    protected $db;

    /** @var \phpbb\user */
    protected $user;

    /** @var \phpbb\language\language */
    protected $language;

    /** @var string phpBB tables prefix */
    protected $table_prefix;

    /**
     * Constructor
     *
     * @param \phpbb\config\config              $config       Config object
     * @param \phpbb\db\driver\driver_interface $db           Database object
     * @param \phpbb\user                       $user         User object
     * @param \phpbb\language\language          $language     Language object
     * @param string                            $table_prefix Tables prefix
     */
    public function __construct(\phpbb\config\config $config, \phpbb\db\driver\driver_interface $db, \phpbb\user $user, \phpbb\language\language $language, $table_prefix)
    {
        $this->config = $config;
        $this->db = $db;
        $this->user = $user;
        $this->language = $language;
        //$this->table_prefix = $table_prefix;
		$this->table_prefix = $this->config['table_prefix'];
    }

    /**
     * Log a search query
     *
     * @param string $search_query The search query
     * @return bool Success or failure
     */
    public function log_search($search_query)
    {
        // Check if logging is enabled
        if (!$this->config['searchlogger_enabled'])
        {
            return false;
        }
        
        // Don't log empty queries
        if (empty($search_query) || trim($search_query) === '')
        {
            return false;
        }

        try
        {
            // Delete old logs based on retention period if needed
            // Only attempt to prune logs occasionally (5% chance) to reduce overhead
            if (rand(1, 100) <= 5) {
                $this->prune_logs();
            }
            return true;
        }
        catch (\Exception $e)
        {
            // Log the error silently - we don't want to disrupt the user's search experience
            // due to logging failures
            if (defined('DEBUG'))
            {
                // Only show error in debug mode
                trigger_error('SearchLogger: ' . $e->getMessage(), E_USER_NOTICE);
            }
            return false;
        }
    }

    /**
     * Prune old logs based on retention period
     *
     * @return bool Success or failure
     */
    protected function prune_logs()
    {
        try
        {
            // Get retention days setting
            $retention_days = (int) $this->config['searchlogger_retention_days'];
            
            // Skip pruning if retention days is set to 0 (meaning keep logs indefinitely)
            if ($retention_days === 0)
            {
                return true;
            }
            
            // Use default value if invalid
            if ($retention_days < 0)
            {
                $retention_days = 30; // Default to 30 days if not set or invalid
            }

            // Calculate cutoff timestamp for pruning
            $prune_time = time() - ($retention_days * 86400); // 86400 = seconds in a day

			// Check if there are any logs older than the prune time
			$sql_count = 'SELECT COUNT(log_id) AS total
                    FROM ' . $this->table_prefix . 'search_logs
                    WHERE search_time < ' . (int) $prune_time;
			$result = $this->db->sql_query($sql_count);
			$row = $this->db->sql_fetchrow($result);
			$this->db->sql_freeresult($result);

			if ($row['total'] > 0)
			{
				// Build and execute query with proper error handling
				$sql = 'DELETE FROM ' . $this->table_prefix . 'search_logs
						WHERE search_time < ' . (int) $prune_time;
				$this->db->sql_query($sql);
			}

            return true;
        }
        catch (\Exception $e)
        {
            // Log the error silently - we don't want to disrupt the user's search experience
            if (defined('DEBUG'))
            {
                // Only show error in debug mode
                trigger_error('SearchLogger pruning: ' . $e->getMessage(), E_USER_NOTICE);
            }
            return false;
        }
    }
}
