<?php
/**
 * Search Logger - Event Listener
 *
 * @package   SearchLogger
 * @copyright (c) 2025 salvocortesiano
 * @license   GNU General Public License, version 2 (GPL-2.0)
 */

namespace salvocortesiano\searchlogger\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event listener
 */
class listener implements EventSubscriberInterface
{
	/** @var \phpbb\config\config */
    protected $config;
	
    /** @var \phpbb\db\driver\driver_interface */
    protected $db;

    /** @var \phpbb\user */
    protected $user;

	/** @var \salvocortesiano\searchlogger\controller\main */
    protected $controller;

    /** @var string phpBB tables prefix */
    protected $table_prefix;

    /**
     * Constructor
     *
     * @param \salvocortesiano\searchlogger\controller\main $controller Main controller
     */
    public function __construct(\phpbb\config\config $config, \phpbb\db\driver\driver_interface $db, \phpbb\user $user, \salvocortesiano\searchlogger\controller\main $controller, $table_prefix)
    {
		$this->config = $config;
        $this->db = $db;
        $this->user = $user;
		$this->controller = $controller;
        $this->table_prefix = $table_prefix;
    }

    /**
     * Assign functions defined in this class to event listeners in the core
     *
     * @return array
     */
    public static function getSubscribedEvents()
    {
        return array(
			'core.search_modify_submit_parameters' 				=> 'log_search',
            'core.search_get_posts_after'      					=> 'log_search',
            'core.search_get_topic_data_after' 					=> 'log_search',
            'core.search_mysql_keywords_main_query_before' 		=> 'log_search',
            'core.search_native_keywords_count_query_before' 	=> 'log_search',
        );
    }
	
    /**
     * Log search queries
     *
     * @param \phpbb\event\data $event Event data
     */
    public function log_search($event)
    {
        try
        {
			if (!$this->config['searchlogger_enabled'])
			{
				return;
			}
            // Get search keywords
            $search_query = '';
            
            if ($event->offsetExists('search_keywords'))
            {
                $search_query = $event->offsetGet('search_keywords');
                // Avoid SQL injection or malformed data by limiting length
                $search_query = substr(trim($search_query), 0, 255);
            }
            else if ($event->offsetExists('keywords'))
            {
                $search_query = $event->offsetGet('keywords');
                // Avoid SQL injection or malformed data by limiting length
                $search_query = substr(trim($search_query), 0, 255);
            }
            else if ($event->offsetExists('ex_fid_ary') && $event->offsetExists('author_ary'))
            {
                // Handle search by author
                $author_ary = $event->offsetGet('author_ary');
                if (!empty($author_ary) && is_array($author_ary))
                {
                    // Safely implode array with proper escaping
                    $cleaned_authors = array_map(function($item) {
                        // Clean and escape author names
                        return htmlspecialchars(trim($item), ENT_QUOTES);
                    }, $author_ary);
                    $search_query = '[Authors: ' . implode(', ', $cleaned_authors) . ']';
                }
                else if (!empty($author_ary) && !is_array($author_ary))
                {
                    // In case author_ary isn't an array as expected
                    $search_query = '[Author: ' . htmlspecialchars(trim($author_ary), ENT_QUOTES) . ']';
                }
            }
            // Don't log empty searches
            if (!empty($search_query))
            {
                // Truncate to reasonable length if somehow we still have a long string
                if (strlen($search_query) > 1000) {
                    $search_query = substr($search_query, 0, 997) . '...';
                }
				$sql_array = array(
					'user_id'         	=> (int) $this->user->data['user_id'],
					'search_query' 		=> utf8_normalize_nfc($search_query),
					'search_time'     	=> time(),
				);
					$sql = 'INSERT INTO ' . $this->table_prefix . 'search_logs ' . $this->db->sql_build_array('INSERT', $sql_array);
					$this->db->sql_query($sql);
                $this->controller->log_search($search_query);
            }
        }
        catch (\Exception $e)
        {
            // Silent exception handling to prevent disruption of search functionality
            // We don't want logging failures to impact the user experience
            if (defined('DEBUG'))
            {
                // Only show error in debug mode
                trigger_error('SearchLogger listener: ' . $e->getMessage(), E_USER_NOTICE);
            }
        }
    }
}
